﻿using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http.Extensions;
using Microsoft.AspNetCore.Mvc;
using Microsoft.IdentityModel.Tokens;
using System;
using System.Collections.Generic;
using System.IdentityModel.Tokens.Jwt;
using System.Security.Claims;
using System.Text;
using System.Threading.Tasks;

using Newtonsoft.Json;

namespace TestAuthorizationServer.Controllers
{
    /// <summary>
    /// OAUTH 컨트롤러
    /// </summary>
    public class OAuthController : Controller
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 로그인 페이지 처리하기 - Login(client_id, scope, response_type, redirect_uri, state)

        /// <summary>
        /// 로그인 페이지 처리하기
        /// </summary>
        /// <param name="client_id">클라이언트 ID</param>
        /// <param name="scope">범위</param>
        /// <param name="response_type">인증 플로우 타입</param>
        /// <param name="redirect_uri">재전송 URI</param>
        /// <param name="state">상태</param>
        /// <returns>액션 결과</returns>
        [HttpGet]
        public IActionResult Login(string client_id, string scope, string response_type, string redirect_uri, string state)
        {
            QueryBuilder queryBuilder = new QueryBuilder();

            queryBuilder.Add("redirectURI", redirect_uri);
            queryBuilder.Add("state"      , state       );

            return View(model : queryBuilder.ToString());
        }

        #endregion
        #region 로그인 페이지 처리하기 - Login(userName, password, redirectURI, state)

        /// <summary>
        /// 로그인 페이지 처리하기
        /// </summary>
        /// <param name="userName">사용자명</param>
        /// <param name="password">패스워드</param>
        /// <param name="redirectURI">재전송 URI</param>
        /// <param name="state">상태</param>
        /// <returns>액션 결과</returns>
        [HttpPost]
        public IActionResult Login(string userName, string password, string redirectURI, string state)
        {
            const string code = "TESTCODE"; // 사용자명과 패스워드에 따라 code를 설정한다.

            QueryBuilder queryBuilder = new QueryBuilder();

            queryBuilder.Add("code" , code );
            queryBuilder.Add("state", state);

            return Redirect($"{redirectURI}{queryBuilder.ToString()}");
        }

        #endregion
        #region 토큰 페이지 처리하기 - Token(grant_type, code, redirect_uri, client_id)

        /// <summary>
        /// 토큰 페이지 처리하기
        /// </summary>
        /// <param name="grant_type">그랜트 타입</param>
        /// <param name="code">코드</param>
        /// <param name="redirect_uri">재전송 URI</param>
        /// <param name="client_id">클라이언트 ID</param>
        /// <returns></returns>
        public async Task<IActionResult> Token(string grant_type, string code, string redirect_uri, string client_id)
        {
            // code 값에 따라 JWT를 발행한다.

            List<Claim> claimList = new List<Claim>()
            {
                new Claim(JwtRegisteredClaimNames.Sub      , "ID0001"       ),
                new Claim(JwtRegisteredClaimNames.Birthdate, "1990-01-01"   ),
                new Claim(JwtRegisteredClaimNames.Email    , "test@daum.net"),
                new Claim(JwtRegisteredClaimNames.Gender   , "Male"         ),
                new Claim("CompanyGroup"                   , "영업1그룹"    ),
                new Claim("CompanyDepartment"              , "영업1팀"      ),
                new Claim("CompanyTitle"                   , "대리"         ),
                new Claim("EmployeeID"                     , "EMP0001"      ),
                new Claim("EmployeeName"                   , "홍길동"       )
            };

            byte[] passwordByteArray = Encoding.UTF8.GetBytes(ConstantHelper.Password);

            SymmetricSecurityKey key = new SymmetricSecurityKey(passwordByteArray);

            string algorithm = SecurityAlgorithms.HmacSha256;

            SigningCredentials signingCredentials = new SigningCredentials(key, algorithm);

            JwtSecurityToken token = new JwtSecurityToken
            (
                ConstantHelper.Issuer,
                ConstantHelper.Audiance,
                claimList,
                notBefore : DateTime.Now,
                expires : DateTime.Now.AddHours(1),
                signingCredentials
            );

            string access_token = new JwtSecurityTokenHandler().WriteToken(token);

            var responseObject = new 
            {
                access_token,
                token_type = "Bearer",
                raw_claim  = "oauthTurorial"
            };

            string responseJSON = JsonConvert.SerializeObject(responseObject);

            byte[] responseByteArray = Encoding.UTF8.GetBytes(responseJSON);

            await Response.Body.WriteAsync(responseByteArray, 0, responseByteArray.Length);

            return Redirect(redirect_uri);
        }

        #endregion
        #region 검증 페이지 처리하기 - Validate()

        /// <summary>
        /// 검증 페이지 처리하기
        /// </summary>
        /// <returns>액션 결과</returns>
        [Authorize]
        public IActionResult Validate()
        {
            if(HttpContext.Request.Query.TryGetValue("access_token", out var accessToken))
            {
                return Ok();
            }

            return BadRequest();
        }

        #endregion
    }
}