﻿using Microsoft.AspNetCore.Authentication;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using System.Net.Http;
using System.Threading.Tasks;

namespace TestClient.Controllers
{
    /// <summary>
    /// 홈 컨트롤러
    /// </summary>
    public class HomeController : Controller
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// HTTP 클라이언트
        /// </summary>
        private readonly HttpClient client;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - HomeController(factory)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="factory">HTTP 클라이언트 팩토리</param>
        public HomeController(IHttpClientFactory factory)
        {
            this.client = factory.CreateClient();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 인덱스 페이치 처리하기 - Index()

        /// <summary>
        /// 인덱스 페이치 처리하기
        /// </summary>
        /// <returns>액션 결과</returns>
        public IActionResult Index()
        {
            return View();
        }

        #endregion
        #region 비밀 페이지 처리하기 - Secret()

        /// <summary>
        /// 비밀 페이지 처리하기
        /// </summary>
        /// <returns>액션 결과</returns>
        [Authorize]
        public async Task<IActionResult> Secret()
        {
            var token = await HttpContext.GetTokenAsync("access_token");

            this.client.DefaultRequestHeaders.Add("Authorization", $"Bearer {token}");

            string authorizationServerURL = "https://localhost:44300/secret/index";

            HttpResponseMessage authorizationServerResponse = await this.client.GetAsync(authorizationServerURL);

            if(authorizationServerResponse.IsSuccessStatusCode)
            {
                string result = await authorizationServerResponse.Content.ReadAsAsync<string>();

                ViewData["AuthorizationServerData"] = result;
            }

            string apiServerURL = "https://localhost:44310/secret/index";

            HttpResponseMessage apiServerResponse = await this.client.GetAsync(apiServerURL);

            if(apiServerResponse.IsSuccessStatusCode)
            {
                string result = await apiServerResponse.Content.ReadAsAsync<string>();

                ViewData["APIServerData"] = result;
            }

            return View();
        }

        #endregion
    }
}