using Microsoft.AspNetCore.Authentication.OAuth;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using System.Collections.Generic;
using System.Security.Claims;
using System.Text;
using System.Threading.Tasks;

using Newtonsoft.Json;

using IdentityModel;

namespace TestClient
{
    /// <summary>
    /// 
    /// </summary>
    public class Startup
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region  ÷ ϱ - ConfigureServices(services)

        /// <summary>
        ///  ÷ ϱ
        /// </summary>
        /// <param name="services"> ÷</param>
        public void ConfigureServices(IServiceCollection services)
        {
            services.AddAuthentication
            (
                options =>
                {
                    options.DefaultAuthenticateScheme = "TestClientCookie";
                    options.DefaultSignInScheme       = "TestClientCookie";
                    options.DefaultChallengeScheme    = "TestServer";
                }
            )
            .AddCookie("TestClientCookie")
            .AddOAuth
            (
                "TestServer",
                options =>
                {
                    options.ClientId              = "CLIENTID0001";
                    options.ClientSecret          = "CLIENTSECRET0001";
                    options.CallbackPath          = "/oauth/callback";
                    options.AuthorizationEndpoint = "https://localhost:44300/oauth/login";
                    options.TokenEndpoint         = "https://localhost:44300/oauth/token";
                    options.SaveTokens            = true;
                    options.Events                = new OAuthEvents()
                    {
                        OnCreatingTicket = context =>
                        {
                            string accessToken      = context.AccessToken;
                            string payload          = accessToken.Split('.')[1];
                            byte[] payloadByteArray = Base64Url.Decode(payload);
                            string json             = Encoding.UTF8.GetString(payloadByteArray);

                            Dictionary<string, string> claimDictionary = JsonConvert.DeserializeObject<Dictionary<string, string>>(json);

                            foreach(KeyValuePair<string, string> claimKeyValuePair in claimDictionary)
                            {
                                context.Identity.AddClaim(new Claim(claimKeyValuePair.Key, claimKeyValuePair.Value));
                            }

                            return Task.CompletedTask;
                        }
                    };
                }
            );

            services.AddHttpClient();

            services.AddControllersWithViews()
                .AddRazorRuntimeCompilation();
        }

        #endregion
        #region ϱ - Configure(app, environment)

        /// <summary>
        /// ϱ
        /// </summary>
        /// <param name="app">ø̼ </param>
        /// <param name="environment"> ȣƮ ȯ</param>
        public void Configure(IApplicationBuilder app, IWebHostEnvironment environment)
        {
            if(environment.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            app.UseRouting();

            app.UseAuthentication();

            app.UseAuthorization();

            app.UseEndpoints
            (
                endpoints =>
                {
                    endpoints.MapDefaultControllerRoute();
                }
            );
        }

        #endregion
    }
}